Imports System.Drawing.Drawing2D
''' <summary>
''' Ein benutzerdefinierter ToolStripProfessionalRenderer mit Farbrotation
''' </summary>

Public Class ColorWheelToolStripRenderer
    Inherits ToolStripProfessionalRenderer

    ''' <summary>
    ''' Event bei nderung
    ''' </summary>
    ''' <remarks>fhrt zum Neu-Zeichnen abhngiger Controls</remarks>
    Public Event Changed()

#Region " Private Felder "

    ' Farbtabelle
    Private WithEvents m_Table As New ColorWheelProfessionalTable

    ' Pens mit Farben der Farbtabelle, die mehrfach gebraucht werden knnen
    Private m_SeparatorLightPen As Pen
    Private m_SeparatorDarkPen As Pen
    Private m_ToolStripBorderPen As Pen
    Private m_MenuBorderPen As Pen
    Private m_MenuItemBorderPen As Pen

    Private m_MouseNormalColors As BiColor
    Private m_MouseHooveredColors As BiColor
    Private m_MouseSelectedColors As BiColor
    Private m_MouseSelectedHooveredColors As BiColor
    Private m_MouseDownColors As BiColor
#End Region

#Region " Konstruktoren "

    Public Sub New(ByVal Table As ColorWheelProfessionalTable)
        MyBase.New(Table)
        m_Table = Table
        ' Pens erstellen
        RefreshPens()
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' gibt den Basis-Farbton der ColorWheelProfessionalTable fr die einfache 
    ''' Control-Hintergrundfarbe zurck oder legt diesen fest
    ''' </summary>
    ''' <remarks>Wert zwischen 0 und 1</remarks>
    Property ControlHue() As Double
        Get
            Return m_Table.ControlHue
        End Get
        Set(ByVal value As Double)
            m_Table.ControlHue = value
        End Set
    End Property


    ''' <summary>
    ''' gibt den Dehnungsfaktor fr die Farben der ColorWheelProfessionalTable untereinander
    ''' zurck oder legt diesen fest
    ''' </summary>
    ''' <remarks>Wert zwischen -1 und +1</remarks>
    Property StretchFactor() As Decimal
        Get
            Return m_Table.StretchFactor
        End Get
        Set(ByVal value As Decimal)
            m_Table.StretchFactor = value
        End Set
    End Property

    ' gibt die Basis-Farbtabelle der ColorWheelProfessionalTable zurck
    ReadOnly Property BaseTable() As ProfessionalColorTable
        Get
            Return m_Table.BaseTable
        End Get
    End Property

    ' legt Farbwert und Dehnungsfaktor der ColorWheelProfessionalTable neu fest
    Public Sub SetHueAndStretchFactor(ByVal ControlHue As Double, ByVal StretchFactor As Decimal)
        m_Table.SetHueAndStretchFactor(ControlHue, StretchFactor)
    End Sub


    ' Pens mit Farben der Farbtabelle, die hufiger gebraucht werden
    ReadOnly Property SeparatorLightPen() As Pen
        Get
            Return m_SeparatorLightPen
        End Get
    End Property
    ReadOnly Property SeparatorDarkPen() As Pen
        Get
            Return m_SeparatorDarkPen
        End Get
    End Property
    ReadOnly Property ToolStripBorderPen() As Pen
        Get
            Return m_ToolStripBorderPen
        End Get
    End Property
    ReadOnly Property MenuBorderPen() As Pen
        Get
            Return m_MenuBorderPen
        End Get
    End Property
    ReadOnly Property MenuItemBorderPen() As Pen
        Get
            Return m_MenuItemBorderPen
        End Get
    End Property

    ' Farben der Farbtabelle, die hufiger gebraucht werden
    ''' <summary>
    ''' Standard-Hintergrundfarbe fr mit diesem Renderer gezeichnete Controls
    ''' </summary>
    ReadOnly Property ControlBackColor() As Color
        Get
            Return m_Table.ToolStripGradientMiddle
        End Get
    End Property

#End Region

#Region " Methoden "

    ''' <summary>
    ''' fhrt bei nderung der Farbrotation der Farbtabelle zum Neuerstellen der
    ''' Pens und liefert ein nderungs-Event
    ''' </summary>
    Private Sub m_Table_Changed() Handles m_Table.Changed
        ' Pens neu erstellen
        RefreshPens()
        ' Event liefern
        RaiseEvent Changed()
    End Sub

    ''' <summary>
    ''' Erstellt Pens mit Farben der Farbtabelle neu
    ''' </summary>
    Private Sub RefreshPens()
        m_SeparatorLightPen = New Pen(m_Table.SeparatorLight)
        m_SeparatorDarkPen = New Pen(m_Table.SeparatorDark)
        m_ToolStripBorderPen = New Pen(m_Table.ToolStripBorder)
        m_MenuBorderPen = New Pen(m_Table.MenuBorder)
        m_MenuItemBorderPen = New Pen(m_Table.MenuItemBorder)

        m_MouseNormalColors = New BiColor(Color.FromArgb(0, m_Table.ToolStripGradientMiddle), Color.FromArgb(0, m_Table.ToolStripGradientEnd))
        m_MouseHooveredColors = New BiColor(m_Table.ButtonSelectedGradientBegin, m_Table.ButtonSelectedGradientEnd)
        m_MouseSelectedColors = New BiColor(m_Table.ButtonCheckedGradientBegin, m_Table.ButtonCheckedGradientEnd)
        m_MouseSelectedHooveredColors = New BiColor(SharedFunctions.MidColor(m_Table.ButtonCheckedGradientBegin, m_Table.ButtonSelectedGradientBegin), SharedFunctions.MidColor(m_Table.ButtonCheckedGradientEnd, m_Table.ButtonSelectedGradientEnd))
        m_MouseDownColors = New BiColor(m_Table.ButtonPressedGradientBegin, m_Table.ButtonPressedGradientEnd)
    End Sub

    ''' <summary>
    ''' liefert eine Farbkombination fr einen Button abhngig vom Auswahl-Zustand
    ''' </summary>
    ''' <param name="MouseDown">Mousebutton ist bettigt</param>
    ''' <param name="Hoovered">Mouse befindet sich ber der Flche des Buttons</param>
    ''' <param name="Selected">Der Button ist ein aktiver ausgewhlter Button</param>
    Public Function ButtonColors(ByVal MouseDown As Boolean, ByVal Hoovered As Boolean, ByVal Selected As Boolean) As BiColor
        If MouseDown Then Return m_MouseDownColors

        If Hoovered Then
            If Selected Then
                Return m_MouseSelectedHooveredColors
            Else
                Return m_MouseHooveredColors
            End If
        Else
            If Selected Then
                Return m_MouseSelectedColors
            Else
                Return m_MouseNormalColors
            End If
        End If
    End Function

    ''' <summary>
    ''' berschriebene Methode zum Erstellen von farblich hervorgehobenen Menitems
    ''' </summary>
    Protected Overrides Sub OnRenderMenuItemBackground(ByVal e As System.Windows.Forms.ToolStripItemRenderEventArgs)

        ' nur wenn die Maus sich ber einem Item im Menstrip befindet,
        ' wird dessen Hintergrund neu gezeichnet
        ' beim ContextMenuStrip wird auch das Parent-Item mit selbst gezeichnetem Hintergrund versehen,
        ' beim MenuStrip und beim StatusStrip nicht

        If (e.ToolStrip.GetType Is GetType(ContextMenuStrip) OrElse e.Item.OwnerItem IsNot Nothing) AndAlso e.Item.Selected Then
            ' runden Pfad erstellen
            Using path As GraphicsPath = SharedFunctions.GetRoundPath(2, 1, e.Item.Width - 4, e.Item.Height - 2, 3)
                ' Pfad mit halbtransparentem LinearGradientBrush fllen
                Using Brsh As Brush = New LinearGradientBrush(New Point(0, 2), New Point(0, e.Item.Height - 2), Color.FromArgb(150, m_Table.MenuItemPressedGradientBegin), Color.FromArgb(150, m_Table.MenuItemPressedGradientEnd))
                    e.Graphics.FillPath(Brsh, path)
                End Using
                ' Pfad mit halbtransparentem Pen zeichnen
                Using Pn As New Pen(Color.FromArgb(150, m_Table.MenuItemBorder))
                    e.Graphics.DrawPath(Pn, path)
                End Using
            End Using
        Else
            ' ansonsten rendert die Basisklasse den Hintergrund
            MyBase.OnRenderMenuItemBackground(e)
        End If
    End Sub

#End Region

End Class

