Imports System.Drawing.Drawing2D
''' <summary>
''' Basisklasse zum Zeichnen eines ListItems
''' </summary>
Public MustInherit Class ListPaintItem

#Region " Private Felder "

    ' Referenz zum ListPaintControl
    Private m_Control As ListPaintControlBase
    ' Rechteck, das Position und Gre des zu zeichnenden Items im ListPaintControl enthlt
    Friend m_Rectangle As Rectangle

    ' Flags, die kennzeichnen, ob das aktuelle Zeichen-Item
    ' selektiert oder gehoovert ist oder ein MouseButton gedrckt ist 
    Private m_Selected As Boolean = False
    Private m_Hoovered As Boolean = False
    Private m_MouseDown As Boolean = False

    ' liefert in festgelegten Timer-Intervallen eine aktuelle Farb-Kombination 
    Private WithEvents m_ColTimer As New BiColorTimer
    ' die aktuelle Farb-Kombination, die zum Zeichnen verwendet wird
    Friend m_ActiveColors As BiColor

#End Region

#Region " Konstruktoren "

    ''' <summary>
    ''' Konstruktor der ListPaintItem-Basisklasse
    ''' </summary>
    ''' <param name="Selected">gibt an, ob das ListItem selektiert ist</param>
    Public Sub New(ByVal Selected As Boolean)
        m_Selected = Selected
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Rechteck, das Position und Gre des zu zeichnenden Items im ListPaintControl enthlt
    ''' </summary>
    Overridable Property Rectangle() As Rectangle
        Get
            Return m_Rectangle
        End Get
        Set(ByVal value As Rectangle)
            m_Rectangle = value
        End Set
    End Property

    ''' <summary>
    ''' Referenz zum ListPaintControl
    ''' </summary>
    Property Control() As ListPaintControlBase
        Get
            Return m_Control
        End Get
        Set(ByVal value As ListPaintControlBase)
            m_Control = value
            m_ActiveColors = m_Control.Renderer.ButtonColors(False, False, m_Selected)
        End Set
    End Property

    ''' <summary>
    ''' gibt an, ob das ListPaintItem selektiert ist
    ''' </summary>
    Property Selected() As Boolean
        Get
            Return m_Selected
        End Get
        Set(ByVal value As Boolean)
            If m_Selected <> value Then
                m_Selected = value
                ' Farben ndern
                SetNewState(100)
            End If
        End Set
    End Property

    ''' <summary>
    ''' gibt an, ob das ListPaintItem gehovert ist
    ''' </summary>
    Property Hoovered() As Boolean
        Get
            Return m_Hoovered
        End Get
        Set(ByVal value As Boolean)
            If m_Hoovered <> value Then
                m_Hoovered = value
                ' Farben ndern
                If m_Hoovered Then
                    SetNewState(100)
                Else
                    SetNewState(500)
                End If
            End If
        End Set
    End Property

    ''' <summary>
    ''' gibt an, ob die Maus ber diesem ListPaintItem sich im Down-Zustand befindet
    ''' </summary>
    Property MouseDown() As Boolean
        Get
            Return m_MouseDown
        End Get
        Set(ByVal value As Boolean)
            If m_MouseDown <> value Then
                m_MouseDown = value
                ' Farben ndern
                SetNewState(100)
            End If
        End Set
    End Property

#End Region

#Region " Methoden "

    ''' <summary>
    ''' passt bei nderungen des Renderers die aktuellen Farben an
    ''' </summary>
    Public Sub RefreshColors()
        m_ActiveColors = m_Control.Renderer.ButtonColors(m_MouseDown, m_Hoovered, m_Selected)
    End Sub

    ''' <summary>
    ''' zeichnet den Hintergrund des ListPaintItems
    ''' </summary>
    Public Overridable Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        If e.ClipRectangle.IntersectsWith(Me.Rectangle) Then
            Dim g As Graphics = e.Graphics
            Dim TmpRect As New Rectangle(m_Rectangle.X, m_Rectangle.Top, m_Rectangle.Width - 2, m_Rectangle.Height - 1)
            ' LinearGradientBrush fr Hintergrund-Flche erstellen
            Dim MyBrush As New LinearGradientBrush(TmpRect, m_ActiveColors.Color1, m_ActiveColors.Color2, LinearGradientMode.Vertical)
            ' Hintergrund-Flche mit LinearGradientBrush fllen
            g.FillRectangle(MyBrush, TmpRect)
            ' Brush-Ressource freigeben
            MyBrush.Dispose()
            ' Pen mit Transparenz der Hintergrundfarben fr den Rahmen erstellen
            Dim BorderPen As New Pen(Color.FromArgb(m_ActiveColors.Color1.A, m_Control.Renderer.MenuBorderPen.Color))
            ' Rahmen-Rechteck zeichnen
            g.DrawRectangle(BorderPen, TmpRect)
            ' Pen-Ressource freigeben
            BorderPen.Dispose()
        End If
    End Sub

    ''' <summary>
    ''' dient in erbenden Klassen zum Freigeben von Ressourcen
    ''' </summary>
    Public MustOverride Sub DisposeBitmap()


    ''' <summary>
    ''' startet einen neuen Farbwechsel fr aktuelle MouseDown-, Hoovered- und Selected-Werte
    ''' </summary>
    ''' <param name="MilliSeconds">Zeit in Millisekunden, die der gesamte Prozess dauern soll</param>
    Private Sub SetNewState(ByVal MilliSeconds As Integer)
        ChangeColors(m_Control.Renderer.ButtonColors(m_MouseDown, m_Hoovered, m_Selected), MilliSeconds)
    End Sub

    ''' <summary>
    ''' startet einen neuen Farbwechsel
    ''' </summary>
    ''' <param name="EndColors">Endfarben, die erreicht werden sollen</param>
    ''' <param name="MilliSeconds">Zeit in Millisekunden, die der gesamte Prozess dauern soll</param>
    ''' <remarks>Als Startfarbe wird ActiveColor verwendet</remarks>
    Private Sub ChangeColors(ByVal EndColors As BiColor, ByVal MilliSeconds As Integer)
        m_ColTimer.StopTimer()
        m_ColTimer = New BiColorTimer(New GradientCollection(m_ActiveColors.Color1, EndColors.Color1), New GradientCollection(m_ActiveColors.Color2, EndColors.Color2), MilliSeconds)
        m_ColTimer.StartTimer()
    End Sub

    ''' <summary>
    ''' bernimmt neue Hintergrundfarben nach einem Changed-Event des ColorTimers
    ''' und fhrt zum Neuzeichnen des Item-Rechtecks
    ''' </summary>
    Private Sub m_ColTimer_Colors_Changed() Handles m_ColTimer.Colors_Changed
        m_ActiveColors = m_ColTimer.ActiveColors
        m_Control.Invalidate(m_Rectangle)
    End Sub

#End Region

End Class
