''' <summary>
''' Eine generische sortierte Liste, die absichert, dass immer genau ein Item 
''' ausgewhlt und die Liste immer sortiert ist
''' </summary>
Public MustInherit Class SortedAddRemoveSelectionList(Of T)
    Inherits AddRemoveSelectionList(Of T)

    Private m_Comparer As IComparer(Of T)
    Public Sub New(ByVal Comparer As IComparer(Of T))
        m_Comparer = Comparer
    End Sub

    Public Shadows Sub Add(ByVal item As T)
        MyBase.Add(item)
        m_SelectedItem = item
        Me.Sort(m_Comparer)
        OnChanged()
    End Sub
    Public Shadows Sub AddRange(ByVal collection As IEnumerable(Of T))
        Dim TmpListe As New List(Of T)
        TmpListe.AddRange(collection)
        TmpListe.Sort(m_Comparer)
        MyBase.AddRange(collection)
        Me.Sort(m_Comparer)
        If TmpListe.Count > 0 Then
            m_SelectedItem = TmpListe(0)
            OnChanged()
        End If
    End Sub
    Public Shadows Sub Clear()
        MyBase.Clear()
        m_SelectedItem = Nothing
        OnChanged()
    End Sub
    Public Shadows Sub Insert(ByVal index As Integer, ByVal item As T)
        Add(item)
    End Sub
    Public Shadows Sub InsertRange(ByVal index As Integer, ByVal collection As IEnumerable(Of T))
        AddRange(collection)
    End Sub
    Public Shadows Function Remove(ByVal item As T) As Boolean
        Dim Index As Integer = Me.IndexOf(item)
        If Index > -1 Then
            Dim result As Boolean = MyBase.Remove(item)
            If Index > 0 Then
                m_SelectedItem = Me(Index - 1)
            ElseIf Me.Count > 0 Then
                m_SelectedItem = Me(0)
            Else
                m_SelectedItem = Nothing
            End If
            OnChanged()
            Return result
        End If
        Return False
    End Function
    Public Shadows Sub RemoveAt(ByVal index As Integer)
        MyBase.RemoveAt(index)
        If index > 0 Then
            m_SelectedItem = Me(index - 1)
        ElseIf Me.Count > 0 Then
            m_SelectedItem = Me(0)
        Else
            m_SelectedItem = Nothing
        End If
        OnChanged()
    End Sub

End Class
