Imports System.Drawing.Drawing2D
''' <summary>
''' Zeichnet einen Button mit berblend-Effekten fr die GlassScrollBar
''' </summary>

Public MustInherit Class GlassButtonPainter
    Inherits ScrollBarAreaPainter

#Region " Fields "

    ' Farbensammlung
    Private WithEvents m_Colors As ScrollBarColorCollection

    ' Aktive Vordergrundfarbe des Buttons (kann auch transparent sein)
    Friend m_ActiveColor As Color
    ' Brush mit Farbe ActiveColor 
    Private m_ActiveBrush As SolidBrush

    ' Alpha fr ForeColor des Buttons beim Start
    ' 0 fr transparenten Button, 255 fr sichtbaren Button
    Private m_StartAlpha As Integer = 255

    ' dunkle Farbe fr den Button-Gradienten
    Private m_ButtonDarkColor As Color

    ' weisse Farbschattierung fr Pfeile / Buttongriff
    ' ist bei transparentem Button reinweiss, bei farbigem 
    ' Button eine Mischfarbe aus Weiss und der Button-Vordergrundfarbe
    Friend m_SymbolWhite As Color

    '' Flag - gibt an, ob der Button beim Start transparent ist oder nicht
    'Private m_IsVisibleAtStart As Boolean

    ' Timer, der im Zeitverlauf mehrere Gradient-Farben zum Zeichnen zur Verfgung stellt
    Private WithEvents m_ColTimer As ColorTimer

    ' Brush mit Farbe weiss und aktuellem Alpha
    Private m_WhiteBrush As SolidBrush

    ' Brush mit Farbe weiss, Alpha liegt linear abhngig von aktuellem Alpha zwischen 0 und 100
    Private m_HighlightBrush As SolidBrush

    ' Brush mit grauer Farbe und aktuellem Alpha
    Private m_GrayBrush As SolidBrush

    ' Pen mit deutlich dunklerer Farbe als ActiveColor
    Private m_ButtonBorderPen As Pen

    ' Rechteck - Begrenzung des Buttons
    Friend m_ButtonRect As Rectangle

    ' abgerundeter GraphicsPath auf Basis ButtonRect
    Private m_ButtonPath As GraphicsPath
    ' abgerundeter GraphicsPath auf Basis ButtonRect, 2 Pixel kleiner
    Private m_GrayPath As GraphicsPath

    ' verschiedene Rechtecke zum Zeichnen von farbigen Flchen / Farbverlufen

    ' die folgenden Rechtecke definieren Flchen, die zusammengesetzt eine 3D-Farbflche
    ' ergeben und nacher mit unterschiedlichen Farben gefllt werden
    Private m_RectLT As Rectangle
    Private m_RectRT As Rectangle
    Private m_RectLB As Rectangle
    Private m_RectRB As Rectangle

    Private m_RectL As Rectangle
    Private m_RectR As Rectangle
    Private m_RectB As Rectangle
    Private m_RectT As Rectangle

    Private m_ColorRect As Rectangle

    ' Rechteck, das spter mit halbtransparenem Weiss gefllt wird, um 
    ' dem Button einen Beleuchtungs-Effekt zu verleihen
    Private m_HighlightRect As Rectangle

    ' Brushes zum Fllen der Rechtecke weiter oben
    Private m_BrushLT As PathGradientBrush
    Private m_BrushRT As PathGradientBrush
    Private m_BrushLB As PathGradientBrush
    Private m_BrushRB As PathGradientBrush

    Private m_BrushL As LinearGradientBrush
    Private m_BrushR As LinearGradientBrush
    Private m_BrushB As LinearGradientBrush
    Private m_BrushT As LinearGradientBrush

#End Region

#Region " Konstruktoren "

    ''' <summary>
    ''' Konstruktor fr die Button-Basisklasse der GlassScrollBar
    ''' </summary>
    ''' <param name="Colors">Farbensammlung</param>
    ''' <param name="IsVisibleAtStart">True, wenn Button beim Start sichtbar sein soll</param>

    Public Sub New(ByVal Colors As ScrollBarColorCollection, ByVal IsVisibleAtStart As Boolean)
        m_Colors = Colors

        ' StartAlpha wird bentigt, wenn der Button im Normalzustand "unsichtbar"
        ' sein soll. Dann erhalten alle gezeichneten Farben einen Alpha-Wert = 0
        If Not IsVisibleAtStart Then
            m_StartAlpha = 0
        End If
        m_ActiveColor = Color.FromArgb(m_StartAlpha, m_Colors.ForeColor)


        ' Brushes und Pens instanziieren (spter wird nur noch die Farbe gewechselt)
        m_WhiteBrush = New SolidBrush(Color.White)
        m_GrayBrush = New SolidBrush(Color.White)
        m_ActiveBrush = New SolidBrush(Color.White)
        m_HighlightBrush = New SolidBrush(Color.White)
        m_ButtonBorderPen = New Pen(Color.White)


        ' neuen ColorTimer instanziieren
        m_ColTimer = New ColorTimer(New GradientCollection(Color.White, Color.White), 0)
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Zeichenrechteck
    ''' </summary>
    ''' <remarks>Das Rechteck, das dieser AreaPainter als Flche zum Zeichnen bentigt. (Dieses 
    ''' Rechteck kann von dem der zugeordneten ScrollBar-Area abweichen.)</remarks>
    Public Overrides ReadOnly Property PaintRectangle() As System.Drawing.Rectangle
        Get
            Return Me.Area.Rectangle
        End Get
    End Property

#End Region

#Region " Methoden "

    ''' <summary>
    ''' startet einen neuen Farbwechsel
    ''' </summary>
    ''' <param name="EndColor">Endfarbe, die erreicht werden soll</param>
    ''' <param name="MilliSeconds">Zeit in Millisekunden, die der gesamte Prozess dauern soll</param>
    ''' <remarks>Als Startfarbe wird ActiveColor verwendet</remarks>
    Private Sub ChangeColors(ByVal EndColor As Color, ByVal MilliSeconds As Integer)
        m_ColTimer.StopTimer()
        m_ColTimer = New ColorTimer(New GradientCollection(m_ActiveColor, EndColor), MilliSeconds)
        m_ColTimer.StartTimer()
    End Sub

    ''' <summary>
    ''' tritt ein beim MouseDown-Ereignis ber der zugeordneten Area
    ''' </summary>
    Public Overrides Sub OnMouseDown()
        ' Farbbergang zu SelectionColor
        ChangeColors(m_Colors.SelectionColor, 100)
    End Sub

    ''' <summary>
    ''' tritt ein beim Eintreten der Maus in die zugeordnete Area
    ''' </summary>
    Public Overrides Sub OnMouseEnter()
        ' Farbbergang zu HooverColor
        ChangeColors(m_Colors.HooverColor, 100)
    End Sub

    ''' <summary>
    ''' tritt ein beim Verlassen der zugeordneten Area durch die Maus
    ''' </summary>
    Public Overrides Sub OnMouseLeave()
        ' Farbbergang zu ForeColor
        ChangeColors(m_Colors.ForeColor, 200)
    End Sub

    ''' <summary>
    ''' tritt ein beim MouseUp-Ereignis ber der zugeordneten Area
    ''' </summary>
    Public Overrides Sub OnMouseUp()
        ' Farbbergang zu HooverColor
        ChangeColors(m_Colors.HooverColor, 200)
    End Sub

    ''' <summary>
    ''' macht einen zuvor transparenten Button sichtbar
    ''' </summary>
    Public Sub SetVisible()
        ' Farbbergang zu Visible ForeColor
        ChangeColors(Color.FromArgb(255, m_Colors.ForeColor), 400)
    End Sub

    ''' <summary>
    ''' macht einen sichtbaren Button unsichtbar
    ''' </summary>
    Public Sub SetInVisible()
        ' Farbbergang zu Invisible ForeColor
        ChangeColors(Color.FromArgb(m_StartAlpha, m_Colors.ForeColor), 1000)
    End Sub

    ''' <summary>
    ''' zeichnet die ScrollBarArea neu
    ''' </summary>
    Public Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        Dim g As Graphics = e.Graphics

        g.SmoothingMode = SmoothingMode.AntiAlias
        ' abgerundeten Button-Pfad mit halbtransparentem Weiss ausfllen
        g.FillPath(m_WhiteBrush, m_ButtonPath)
        ' eine etwas kleinere abgerundete Flche mit halbtransparentem Grau ausfllen
        g.FillPath(m_GrayBrush, m_GrayPath)

        ' Eine 3D-Glass-Button-Flche zeichnen
        g.SmoothingMode = SmoothingMode.None
        g.FillRectangle(m_BrushLT, m_RectLT)
        g.FillRectangle(m_BrushRT, m_RectRT)
        g.FillRectangle(m_BrushLB, m_RectLB)
        g.FillRectangle(m_BrushRB, m_RectRB)

        g.FillRectangle(m_BrushL, m_RectL)
        g.FillRectangle(m_BrushR, m_RectR)
        g.FillRectangle(m_BrushB, m_RectB)
        g.FillRectangle(m_BrushT, m_RectT)

        g.FillRectangle(m_ActiveBrush, m_ColorRect)

        ' Highlight zeichnen - als ob von links oder oben Licht auf den Button scheint
        g.FillRectangle(m_HighlightBrush, m_HighlightRect)

        ' dunklen Button-Rahmen zeichnen
        g.SmoothingMode = SmoothingMode.AntiAlias
        g.DrawPath(m_ButtonBorderPen, m_ButtonPath)

        ' Symbol (Griff oder Pfeil) zeichnen
        g.SmoothingMode = SmoothingMode.Default
        PaintSymbol(e)
    End Sub

    ''' <summary>
    ''' ermglicht das Aktualisieren nach nderungen der ScrollBarArea
    ''' </summary>
    Public Overrides Sub Update()
        ' Rechteck zum Zeichnen, davon abhngige Pfade und Brushes neu definieren
        OnRectangleChanged()
    End Sub

    ''' <summary>
    ''' legt vom Rechteck der gezeichneten Area abhngige Pfade und Rechtecke neu fest
    ''' und fhrt zum Neudefinieren positionsabhngiger Brushes
    ''' </summary>
    Private Sub OnRectangleChanged()
        'aus berschreibender Klasse das Begrenzungs-Rechteck fr den Button erhalten 
        m_ButtonRect = GetButtonRectangle()

        ' runde Pfade zum Zeichnen des Button-Hintergrunds
        m_ButtonPath = SharedFunctions.GetRoundPath(m_ButtonRect.X, m_ButtonRect.Y, m_ButtonRect.Width, m_ButtonRect.Height, 2.0F)
        m_GrayPath = SharedFunctions.GetRoundPath(m_ButtonRect.X + 2, m_ButtonRect.Y + 2, m_ButtonRect.Width - 2, m_ButtonRect.Height - 2, 2.0F)

        ' Rechtecke zum Zeichnen des 3D-Glass-Button-Effekts
        Dim ColorRect As New Rectangle(m_ButtonRect.X + 2, m_ButtonRect.Y + 2, m_ButtonRect.Width - 3, m_ButtonRect.Height - 3)

        If Me.Area.Orientation = Windows.Forms.Orientation.Vertical Then
            m_RectL = New Rectangle(ColorRect.X, ColorRect.Y + 2, 5, ColorRect.Height - 4)
            m_RectR = New Rectangle(ColorRect.Right - 5, ColorRect.Y + 2, 5, ColorRect.Height - 4)
            m_RectT = New Rectangle(ColorRect.X + 5, ColorRect.Y, ColorRect.Width - 10, 2)
            m_RectB = New Rectangle(ColorRect.X + 5, ColorRect.Bottom - 2, ColorRect.Width - 10, 2)
            m_HighlightRect = New Rectangle(m_ButtonRect.X + 1, m_ButtonRect.Y + 1, 6, m_ButtonRect.Height - 2)
        Else
            m_RectL = New Rectangle(ColorRect.X, ColorRect.Y + 5, 2, ColorRect.Height - 10)
            m_RectR = New Rectangle(ColorRect.Right - 2, ColorRect.Y + 5, 2, ColorRect.Height - 10)
            m_RectT = New Rectangle(ColorRect.X + 2, ColorRect.Y, ColorRect.Width - 4, 5)
            m_RectB = New Rectangle(ColorRect.X + 2, ColorRect.Bottom - 5, ColorRect.Width - 4, 5)
            m_HighlightRect = New Rectangle(m_ButtonRect.X + 1, m_ButtonRect.Y + 1, m_ButtonRect.Width - 2, 6)
        End If
        m_RectLT = New Rectangle(ColorRect.X, ColorRect.Y, m_RectL.Width, m_RectT.Height)
        m_RectRT = New Rectangle(m_RectR.X, ColorRect.Y, m_RectR.Width, m_RectT.Height)
        m_RectLB = New Rectangle(ColorRect.X, m_RectB.Y, m_RectL.Width, m_RectB.Height)
        m_RectRB = New Rectangle(m_RectR.X, m_RectB.Y, m_RectR.Width, m_RectB.Height)

        m_ColorRect = New Rectangle(m_RectL.Right, m_RectT.Bottom, m_RectR.Left - m_RectL.Right, m_RectB.Top - m_RectT.Bottom)

        ' Pfad- und Rechteck-abhngige Brushes und Vordergrund-Symbol neu definieren
        GetRectangleDependentBrushes()
    End Sub

    ''' <summary>
    ''' liefert einen Misch-Farbton zwischen Weiss und ActiveColor
    ''' </summary>
    ''' <returns>Color-Struktur</returns>
    Private Function GetSymbolWhite() As Color
        ' diese Funktion liefert die weisse Farbe fr die Symbole, die auf der 
        ' Button-Oberflche gezeichnet werden. Bei unsichtbarem Button (Pfeile oben und unten)
        ' ist das normales Weiss. Bei sichtbarem Button (gehoovert oder selected) mchte
        ' ich erreichen, dass das Symbol sich nicht durch zu starkes Weiss vom Hintergrund 
        ' abhebt. Also erzeuge ich eine Mischfarbe zwischen Weiss und der eigentlichen
        ' Button-Vordergrund-Farbe.
        ' Wenn fr RDiff =, GDiff =... auf / 4.0F verzichtet werden wrde,
        ' wrde sich ergeben:
        ' -  bei Alpha 255 -> ActiveColor
        ' -  bei Alpha 0 -> White
        Dim Alpha As Single = m_ActiveColor.A

        Dim RDiff, GDiff, BDiff As Single
        Dim RVal, GVal, BVal As Integer
        RDiff = (255 - m_ActiveColor.R) / 4.0F
        GDiff = (255 - m_ActiveColor.G) / 4.0F
        BDiff = (255 - m_ActiveColor.B) / 4.0F

        RVal = Math.Min(255.0F - Math.Round(Alpha * RDiff / 255.0F), 255)
        GVal = Math.Min(255.0F - Math.Round(Alpha * GDiff / 255.0F), 255)
        BVal = Math.Min(255.0F - Math.Round(Alpha * BDiff / 255.0F), 255)

        Return Color.FromArgb(RVal, GVal, BVal)
    End Function

    ''' <summary>
    ''' erstellt alle verwendeten Grundfarben und Brushes neu
    ''' </summary>
    Private Sub GetAllBrushes()
        ' Hauptfarbe des Buttons
        m_ActiveBrush.Color = m_ActiveColor

        ' Weissfarbe fr die Button-Symbole
        m_SymbolWhite = GetSymbolWhite()

        Dim Alpha As Byte = m_ActiveColor.A

        ' HighlightBrush ist der Brush zum Zeichnen des Licht-Rechtecks
        Dim HighlightAlpha As Integer = Math.Round(CSng(Alpha) * 100.0F / 255.0F)
        m_HighlightBrush.Color = Color.FromArgb(HighlightAlpha, Color.White)

        ' WhiteBrush ist ein Brush zum Zeichnen des Button-Hintergrunds
        m_WhiteBrush.Color = Color.FromArgb(Alpha, Color.White)

        Dim Original As New HSLColor(m_ActiveColor)
        Dim HSL As HSLColor

        ' Farbe fr den Button-Rahmen
        HSL = New HSLColor(Original)
        HSL.Lightness -= 35.0R / 100.0R
        m_ButtonBorderPen.Color = Color.FromArgb(Alpha, HSL.ToRGB)

        ' dunklere Farbe fr den 3D-Glass-Effekt
        HSL = New HSLColor(Original)
        HSL.Lightness -= 10.0R / 100.0R
        m_ButtonDarkColor = Color.FromArgb(Alpha, HSL.ToRGB)

        ' habltransparente graue Farbe zum Zeichnen des Button-Hintergrunds
        m_GrayBrush.Color = Color.FromArgb(Alpha, Color.FromArgb(221, 221, 221))

        ' Pfad- und Rechteck-abhngige Brushes und Vordergrund-Symbol neu definieren
        GetRectangleDependentBrushes()
    End Sub

    ''' <summary>
    ''' definiert Pfad- und Rechteck-abhngige Brushes sowie das Vordergrund-Symbol neu
    ''' </summary>

    Private Sub GetRectangleDependentBrushes()

        Dim WhiteCol As Color = Color.FromArgb(m_ActiveColor.A, Color.White)

        m_BrushL = GetLinearBrush(m_RectL, WhiteCol, m_ActiveColor, LinearGradientMode.Horizontal)
        m_BrushR = GetLinearBrush(m_RectR, m_ActiveColor, m_ButtonDarkColor, LinearGradientMode.Horizontal)
        m_BrushT = GetLinearBrush(m_RectT, WhiteCol, m_ActiveColor, LinearGradientMode.Vertical)
        m_BrushB = GetLinearBrush(m_RectB, m_ActiveColor, m_ButtonDarkColor, LinearGradientMode.Vertical)

        m_BrushLT = GetCornerBrush(m_RectLT, Corners.LeftTop, WhiteCol, m_ButtonDarkColor, m_ActiveColor)
        m_BrushRT = GetCornerBrush(m_RectRT, Corners.RightTop, WhiteCol, m_ButtonDarkColor, m_ActiveColor)
        m_BrushLB = GetCornerBrush(m_RectLB, Corners.LeftBottom, WhiteCol, m_ButtonDarkColor, m_ActiveColor)
        m_BrushRB = GetCornerBrush(m_RectRB, Corners.RightBottom, WhiteCol, m_ButtonDarkColor, m_ActiveColor)

        ' Vordergrund-Symbol neu definieren
        GetSymbol()
    End Sub


    ''' <summary>
    ''' definiert das Vordergrund-Symbol neu
    ''' </summary>
    MustOverride Sub GetSymbol()


    ''' <summary>
    ''' diefert das Begrenzungsrechteck fr den Button 
    ''' </summary>
    MustOverride Function GetButtonRectangle() As Rectangle

    ''' <summary>
    ''' zeichnet das Symbol auf dem Button neu
    ''' </summary>
    ''' <param name="e">PaintEventArgs</param>
    MustOverride Sub PaintSymbol(ByVal e As System.Windows.Forms.PaintEventArgs)


    ''' <summary>
    ''' Enumeration zur Bezeichnung von Ecken eines Rechtecks
    ''' </summary>
    Private Enum Corners
        LeftTop
        RightTop
        LeftBottom
        RightBottom
    End Enum

    ''' <summary>
    ''' Liefert einen PathGradientBrush zum Zeichnen eine 3D-Glass-Buttons
    ''' </summary>
    ''' <param name="Rect">Rechteck-Begrenzung</param>
    ''' <param name="Corner">Ecke</param>
    ''' <param name="WhiteCol">Weiss-Farbe</param>
    ''' <param name="DarkCol">Dunkle Farbe</param>
    ''' <param name="ButtonCol">Button-Farbe</param>
    ''' <returns>PathGradientBrush</returns>
    Private Function GetCornerBrush(ByVal Rect As Rectangle, ByVal Corner As Corners, ByVal WhiteCol As Color, ByVal DarkCol As Color, ByVal ButtonCol As Color) As PathGradientBrush
        ' temporres Rechteck fr GraphicsPath
        Dim TmpRect As New RectangleF(CSng(Rect.X) - 1.0F, CSng(Rect.Y) - 1.0F, CSng(Rect.Width) + 1.0F, CSng(Rect.Height) + 1.0F)
        ' GraphicsPath mit temporrem Rechteck erstellen 
        Dim GP As New GraphicsPath
        GP.AddRectangle(TmpRect)
        GP.CloseAllFigures()

        ' PathGradientBrush auf Grundlage des GraphicsPath erstellen
        Dim PBrush As New PathGradientBrush(GP)
        Dim cols(3) As Color
        ' zentrale Farbe 
        PBrush.CenterColor = ButtonCol

        Select Case Corner
            Case Corners.LeftTop
                ' Ecke links oben
                cols(0) = WhiteCol
                cols(1) = WhiteCol
                cols(2) = ButtonCol
                cols(3) = WhiteCol
                PBrush.CenterPoint = New PointF(TmpRect.Right, TmpRect.Bottom)
            Case Corners.RightTop
                ' Ecke rechts oben
                cols(0) = WhiteCol
                cols(1) = WhiteCol
                cols(2) = DarkCol
                cols(3) = ButtonCol
                PBrush.CenterPoint = New PointF(TmpRect.Left + 1.0F, TmpRect.Bottom)
            Case Corners.RightBottom
                ' Ecke rechts unten
                cols(0) = ButtonCol
                cols(1) = DarkCol
                cols(2) = DarkCol
                cols(3) = DarkCol
                PBrush.CenterPoint = New PointF(TmpRect.Left + 1.0F, TmpRect.Top + 1.0F)
            Case Corners.LeftBottom
                ' Ecke links unten
                cols(0) = WhiteCol
                cols(1) = ButtonCol
                cols(2) = DarkCol
                cols(3) = WhiteCol
                PBrush.CenterPoint = New PointF(TmpRect.Right, TmpRect.Top + 1.0F)
        End Select

        ' Umgebungsfarben fr den PathgradientBrush festlegen
        PBrush.SurroundColors = cols
        Return PBrush
    End Function

    ''' <summary>
    ''' Liefert einen LinearGradientBrush fr festgelegte Farben zum Ausfllen eines Rechtecks
    ''' </summary>
    ''' <param name="Rect">Begrenzungs-Rechteck zum Zeichnen</param>
    ''' <param name="Color1">Color-Struktur</param>
    ''' <param name="Color2">Color-Struktur</param>
    ''' <param name="Mode">LinearGradientMode</param>
    ''' <returns>LinearGradientBrush</returns>
    Private Function GetLinearBrush(ByVal Rect As Rectangle, ByVal Color1 As Color, ByVal Color2 As Color, ByVal Mode As LinearGradientMode)
        Dim TmpRect As New Rectangle(Rect.X - 1, Rect.Y - 1, Rect.Width + 2, Rect.Height + 2)
        Return New LinearGradientBrush(TmpRect, Color1, Color2, Mode)
    End Function

    ''' <summary>
    ''' Eventhandler fr das Color_Changed-Event des ColorTimers
    ''' </summary>
    Private Sub m_ColTimer_Color_Changed() Handles m_ColTimer.Color_Changed
        ' nach jedem Changed-Event des ColorTimers liefert dieser eine neue Aktive Farbe
        ' Diese wird dann neue Hauptfarbe zum Zeichnen des Buttons
        m_ActiveColor = m_ColTimer.ActiveColor
        ' alle Brushes neu erstellen
        GetAllBrushes()
        ' ScrollBar zum Neuzeichnen der Area-Flche zwingen
        Me.Area.Invalidate(Me.Area.Rectangle)
    End Sub

    ''' <summary>
    ''' Eventhandler fr das Changed-Event der Farbensammlung
    ''' </summary>
    Private Sub m_Colors_Changed() Handles m_Colors.Changed
        ' nach diesem Event (findet normalerweise nur bei Initialisierung des Buttons bzw. im DesignMode
        ' des Windows Forms Editors statt - und nicht gerade in gehoovertem oder selektiertem Zustand)
        ' wird nur die Aktive Farbe des Buttons gewechselt 
        m_ActiveColor = Color.FromArgb(m_StartAlpha, m_Colors.ForeColor)
        ' danach werden alle verwendeten Brushes aktualisiert
        GetAllBrushes()
        ' (in der weiteren Folge wird die ganze ScrollBar nach Invalidate neu gezeichnet)
    End Sub

#End Region

End Class
