''' <summary>
''' zeichnet eine ScrollBar in Vista-hnlichem Stil
''' </summary>

Public Class GlassScrollBarPainter
    Inherits ScrollBarPainter

#Region " Private Felder "

    ' Farbensammlung zum Zeichnen der ScrollBar-Inhalte
    Private m_ScrollBarColors As ScrollBarColorCollection

    ' AreaPainter, die die einzelnen Areas auf der ScrollBar zeichnen
    Private m_AreaButtonTopPainter As GlassArrowButtonPainter
    Private m_AreaButtonBottomPainter As GlassArrowButtonPainter
    Private m_AreaLargeScrollTopPainter As ScrollBarAreaPainter
    Private m_AreaLargeScrollBottomPainter As ScrollBarAreaPainter
    Private m_AreaButtonScrollPainter As ScrollBarAreaPainter
    Private WithEvents m_AreaBackgroundPainter As GlassScrollBarBackgroundPainter

#End Region

#Region " Konstruktoren "

    ''' <summary>
    ''' Konstruktor fr einen neuen GlassScrollBarPainter
    ''' </summary>
    Public Sub New()
        ' Farbensammlung und AreaPainter initialisieren
        m_ScrollBarColors = New ScrollBarColorCollection(Color.Blue, Color.WhiteSmoke, Color.Pink, Color.Red)
        m_AreaButtonTopPainter = New GlassArrowButtonPainter(m_ScrollBarColors, False, True)
        m_AreaButtonBottomPainter = New GlassArrowButtonPainter(m_ScrollBarColors, False, False)
        m_AreaLargeScrollTopPainter = New EmptyScrollBarAreaPainter
        m_AreaLargeScrollBottomPainter = New EmptyScrollBarAreaPainter
        m_AreaButtonScrollPainter = New GlassScrollButtonPainter(m_ScrollBarColors, True)
        m_AreaBackgroundPainter = New GlassScrollBarBackgroundPainter(m_ScrollBarColors)
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' AreaPainter fr den Hintergrund des Controls
    ''' </summary>
    Public Overrides ReadOnly Property AreaBackgroundPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaBackgroundPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr den unteren Pfeil-Button
    ''' </summary>
    Public Overrides ReadOnly Property AreaButtonBottomPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaButtonBottomPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr den Scroll-Button
    ''' </summary>
    Public Overrides ReadOnly Property AreaButtonScrollPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaButtonScrollPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr den oberen Pfeil-Button
    ''' </summary>
    Public Overrides ReadOnly Property AreaButtonTopPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaButtonTopPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr die untere Scroll-Flche
    ''' </summary>
    Public Overrides ReadOnly Property AreaLargeScrollBottomPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaLargeScrollBottomPainter
        End Get
    End Property

    ''' <summary>
    ''' AreaPainter fr die obere Scroll-Flche
    ''' </summary>
    Public Overrides ReadOnly Property AreaLargeScrollTopPainter() As ScrollBarAreaPainter
        Get
            Return m_AreaLargeScrollTopPainter
        End Get
    End Property

    ''' <summary>
    ''' Lnge des unteren Pfeil-Buttons
    ''' </summary>
    Public Overrides ReadOnly Property ButtonBottomLength() As Integer
        Get
            Return 18
        End Get
    End Property

    ''' <summary>
    ''' Lnge des oberen Pfeil-Buttons
    ''' </summary>
    Public Overrides ReadOnly Property ButtonTopLength() As Integer
        Get
            Return 18
        End Get
    End Property

    ''' <summary>
    ''' Mindestlnge fr den Scroll-Button
    ''' </summary>
    ''' <value>Ein Wert > 0</value>
    Public Overrides ReadOnly Property MinButtonScrollLength() As Integer
        Get
            Return 15
        End Get
    End Property

    ''' <summary>
    ''' Minimum fr Breite
    ''' </summary>
    Public Overrides ReadOnly Property MinVirtualWidth() As Integer
        Get
            Return 17
        End Get
    End Property

    ''' <summary>
    ''' Standard-Farben der ScrollBar
    ''' </summary>
    Public Overrides ReadOnly Property ScrollBarColors() As ScrollBarColorCollection
        Get
            Return m_ScrollBarColors
        End Get
    End Property

#End Region

#Region " Methoden "

    ''' <summary>
    ''' Eventhandler fr das MouseEnter-Ereigneis des Hintergrund-Painters
    ''' </summary>
    Private Sub m_AreaBackgroundPainter_MouseEnter() Handles m_AreaBackgroundPainter.MouseEnter
        ' bewirkt, dass nach Eintreten der Maus in die ScrollBar 
        ' die Pfeil-Buttons sichtbar werden
        m_AreaButtonTopPainter.SetVisible()
        m_AreaButtonBottomPainter.SetVisible()
    End Sub

    ''' <summary>
    ''' Eventhandler fr das MouseLeave-Ereigneis des Hintergrund-Painters
    ''' </summary>
    Private Sub m_AreaBackgroundPainter_MouseLeave() Handles m_AreaBackgroundPainter.MouseLeave
        ' bewirkt, dass nach Verlassen der ScrollBar durch die Maus 
        ' die Pfeil-Buttons unsichtbar werden
        m_AreaButtonTopPainter.SetInVisible()
        m_AreaButtonBottomPainter.SetInVisible()
    End Sub

#End Region

End Class
